/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: geoextract.c,v 1.2 1995/10/10 14:49:18 deaven Exp $
 */

/* Read geometry files, skipping excess lines or data not recognizable
 * as geometry data, and write all readable geometries to the output.
 */

#include <stdio.h>
#include <string.h>
#include "dmd/geometry.h"
#include "dmd/memory.h"

char *usage = "usage: %s [-v] [-o out-file] [geometry]\n";

void main(int argc, char **argv)
{
	int arg;
	int frames, skipped;
	int verbose = 0;
	int ok;
	Geometry *g;
	char *ifn = "-", *ofn = "-";
	FILE *fin, *fout;
	long offset;

	for(arg=1; arg<argc; ++arg)
	  if(argv[arg][0] == '-')
	    switch(argv[arg][1])
	    {
	      case 'v':
		verbose = 1;
		break;
	      case 'o':
		ofn = argv[++arg];
		break;
	      default:
	      	fprintf(stderr, usage, argv[0]);
	      	exit(1);
	    }
	  else
	    ifn = argv[arg];

	fin = (strcmp(ifn, "-") == 0 ? stdin : fopen(ifn, "r"));
	if(fin == NULL)
	{
		fprintf(stderr, "%s: cannot open %s\n", argv[0], ifn);
		exit(1);
	}

	fout = (strcmp(ofn, "-") == 0 ? stdout : fopen(ofn, "w"));
	if(fout == NULL)
	{
		fprintf(stderr, "%s: cannot open %s\n", argv[0], ofn);
		exit(1);
	}

	frames = skipped = 0;
	ok = 1; /* last attempt succeeded */

	while(1)
	{
	  do {
	    offset = ftell(fin);
	    if((g = GreadWithError(fin, NULL)) == NULL) {
	      if(fseek(fin, offset, SEEK_SET) < 0) {
		perror("seeking on input");
		exit(1);
	      }
	      while(fgetc(fin) != '\n' && !feof(fin));
	      ++skipped;
	      if(verbose && ok) {
		fprintf(stderr, "X");
		fflush(stderr);
		ok = 0;
	      }
	    }
	  } while(g == NULL && !feof(fin));

	  if(feof(fin)) break;

	  Gprint(fout, g);
	  ++frames;
	  if(verbose) {
	    fprintf(stderr, ".");
	    fflush(stderr);
	    ok = 1;
	  }

	  Gfree(g);
	}

	if(fin != stdin) fclose(fin);
	if(fout != stdout) fclose(fout);
	if(verbose) fprintf(stderr, "\n");

	fprintf(stderr, "%s: read %d geometr%s, skipped %d line%s\n", argv[0],
	  frames, frames > 1 ? "ies" : "y",
	  skipped, skipped > 1 ? "s" : "");

	exit(0);
}
